@@ -1761,9 +1761,12 @@ impl<'a> BinaryReader<'a> {
             0x9e => Operator::I16x8ExtMulLowI8x16U,
             0x9f => Operator::I16x8ExtMulHighI8x16U,
             0xa0 => Operator::I32x4Abs,
+            0xa2 => Operator::I8x16SwizzleRelaxed,
             0xa1 => Operator::I32x4Neg,
             0xa3 => Operator::I32x4AllTrue,
             0xa4 => Operator::I32x4Bitmask,
+            0xa5 => Operator::I32x4TruncSatF32x4SRelaxed,
+            0xa6 => Operator::I32x4TruncSatF32x4URelaxed,
             0xa7 => Operator::I32x4ExtendLowI16x8S,
             0xa8 => Operator::I32x4ExtendHighI16x8S,
             0xa9 => Operator::I32x4ExtendLowI16x8U,
@@ -1772,7 +1775,12 @@ impl<'a> BinaryReader<'a> {
             0xac => Operator::I32x4ShrS,
             0xad => Operator::I32x4ShrU,
             0xae => Operator::I32x4Add,
+            0xaf => Operator::F32x4FmaRelaxed,
+            0xb0 => Operator::F32x4FmsRelaxed,
             0xb1 => Operator::I32x4Sub,
+            0xb2 => Operator::I8x16LaneSelect,
+            0xb3 => Operator::I16x8LaneSelect,
+            0xb4 => Operator::F32x4MinRelaxed,
             0xb5 => Operator::I32x4Mul,
             0xb6 => Operator::I32x4MinS,
             0xb7 => Operator::I32x4MinU,
@@ -1787,6 +1795,8 @@ impl<'a> BinaryReader<'a> {
             0xc1 => Operator::I64x2Neg,
             0xc3 => Operator::I64x2AllTrue,
             0xc4 => Operator::I64x2Bitmask,
+            0xc5 => Operator::I32x4TruncSatF64x2SZeroRelaxed,
+            0xc6 => Operator::I32x4TruncSatF64x2UZeroRelaxed,
             0xc7 => Operator::I64x2ExtendLowI32x4S,
             0xc8 => Operator::I64x2ExtendHighI32x4S,
             0xc9 => Operator::I64x2ExtendLowI32x4U,
@@ -1795,7 +1805,12 @@ impl<'a> BinaryReader<'a> {
             0xcc => Operator::I64x2ShrS,
             0xcd => Operator::I64x2ShrU,
             0xce => Operator::I64x2Add,
+            0xcf => Operator::F64x2FmaRelaxed,
+            0xd0 => Operator::F64x2FmsRelaxed,
             0xd1 => Operator::I64x2Sub,
+            0xd2 => Operator::I32x4LaneSelect,
+            0xd3 => Operator::I64x2LaneSelect,
+            0xd4 => Operator::F64x2MinRelaxed,
             0xd5 => Operator::I64x2Mul,
             0xd6 => Operator::I64x2Eq,
             0xd7 => Operator::I64x2Ne,
@@ -1809,6 +1824,7 @@ impl<'a> BinaryReader<'a> {
             0xdf => Operator::I64x2ExtMulHighI32x4U,
             0xe0 => Operator::F32x4Abs,
             0xe1 => Operator::F32x4Neg,
+            0xe2 => Operator::F32x4MaxRelaxed,
             0xe3 => Operator::F32x4Sqrt,
             0xe4 => Operator::F32x4Add,
             0xe5 => Operator::F32x4Sub,
@@ -1820,6 +1836,7 @@ impl<'a> BinaryReader<'a> {
             0xeb => Operator::F32x4PMax,
             0xec => Operator::F64x2Abs,
             0xed => Operator::F64x2Neg,
+            0xee => Operator::F64x2MaxRelaxed,
             0xef => Operator::F64x2Sqrt,
             0xf0 => Operator::F64x2Add,
             0xf1 => Operator::F64x2Sub,
