@@ -119,7 +119,12 @@ enum FrameKind {
 }
 
 impl OperatorValidator {
-    pub fn new(
+    /// Creates a new operator validator which will be used to validate a
+    /// function whose type is the `ty` index specified.
+    ///
+    /// The `resources` are used to learn about the function type underlying
+    /// `ty`.
+    pub fn new_func(
         ty: u32,
         offset: usize,
         features: &WasmFeatures,
@@ -146,6 +151,25 @@ impl OperatorValidator {
         })
     }
 
+    /// Creates a new operator validator which will be used to validate an
+    /// `init_expr` constant expression which should result in the `ty`
+    /// specified.
+    pub fn new_init_expr(features: &WasmFeatures, ty: Type) -> OperatorValidator {
+        OperatorValidator {
+            num_locals: 0,
+            locals: Vec::new(),
+            operands: Vec::new(),
+            control: vec![Frame {
+                kind: FrameKind::Block,
+                block_type: TypeOrFuncType::Type(ty),
+                height: 0,
+                unreachable: false,
+            }],
+            features: *features,
+            br_table_tmp: Vec::new(),
+        }
+    }
+
     pub fn define_locals(&mut self, offset: usize, count: u32, ty: Type) -> Result<()> {
         self.features
             .check_value_type(ty)
@@ -399,6 +423,18 @@ impl OperatorValidator {
         Ok(())
     }
 
+    fn check_relaxed_simd_enabled(&self) -> OperatorValidatorResult<()> {
+        // Relaxed SIMD operators make sense only with SIMD and be non-deterministic.
+        self.check_non_deterministic_enabled()?;
+        self.check_simd_enabled()?;
+        if !self.features.relaxed_simd {
+            return Err(OperatorValidatorError::new(
+                "Relaxed SIMD support is not enabled",
+            ));
+        }
+        Ok(())
+    }
+
     fn check_exceptions_enabled(&self) -> OperatorValidatorResult<()> {
         if !self.features.exceptions {
             return Err(OperatorValidatorError::new(
@@ -629,7 +665,7 @@ impl OperatorValidator {
                 // targets an actual `catch` to get the exception.
                 let (_, kind) = self.jump(relative_depth)?;
                 if kind != FrameKind::Catch && kind != FrameKind::CatchAll {
-                    bail_op_err!("rethrow target was not a `catch` block");
+                    bail_op_err!("invalid rethrow label: target was not a `catch` block");
                 }
                 self.unreachable();
             }
@@ -1358,9 +1394,10 @@ impl OperatorValidator {
             Operator::RefFunc { function_index } => {
                 self.check_reference_types_enabled()?;
                 if resources.type_of_function(function_index).is_none() {
-                    return Err(OperatorValidatorError::new(
-                        "unknown function: function index out of bounds",
-                    ));
+                    return Err(OperatorValidatorError::new(format!(
+                        "unknown function {}: function index out of bounds",
+                        function_index,
+                    )));
                 }
                 if !resources.is_function_referenced(function_index) {
                     return Err(OperatorValidatorError::new("undeclared function reference"));
@@ -1521,6 +1558,19 @@ impl OperatorValidator {
                 self.pop_operand(Some(Type::V128))?;
                 self.push_operand(Type::V128)?;
             }
+            Operator::F32x4FmaRelaxed
+            | Operator::F32x4FmsRelaxed
+            | Operator::F64x2FmaRelaxed
+            | Operator::F64x2FmsRelaxed
+            | Operator::F32x4MinRelaxed
+            | Operator::F32x4MaxRelaxed
+            | Operator::F64x2MinRelaxed
+            | Operator::F64x2MaxRelaxed => {
+                self.check_relaxed_simd_enabled()?;
+                self.pop_operand(Some(Type::V128))?;
+                self.pop_operand(Some(Type::V128))?;
+                self.push_operand(Type::V128)?;
+            }
             Operator::I8x16Eq
             | Operator::I8x16Ne
             | Operator::I8x16LtS
@@ -1635,6 +1685,8 @@ impl OperatorValidator {
             | Operator::F64x2PromoteLowF32x4
             | Operator::F64x2ConvertLowI32x4S
             | Operator::F64x2ConvertLowI32x4U
+            | Operator::I32x4TruncSatF32x4S
+            | Operator::I32x4TruncSatF32x4U
             | Operator::I32x4TruncSatF64x2SZero
             | Operator::I32x4TruncSatF64x2UZero
             | Operator::F32x4ConvertI32x4S
@@ -1654,8 +1706,6 @@ impl OperatorValidator {
             | Operator::I32x4Neg
             | Operator::I64x2Abs
             | Operator::I64x2Neg
-            | Operator::I32x4TruncSatF32x4S
-            | Operator::I32x4TruncSatF32x4U
             | Operator::I16x8ExtendLowI8x16S
             | Operator::I16x8ExtendHighI8x16S
             | Operator::I16x8ExtendLowI8x16U
@@ -1676,6 +1726,14 @@ impl OperatorValidator {
                 self.pop_operand(Some(Type::V128))?;
                 self.push_operand(Type::V128)?;
             }
+            Operator::I32x4TruncSatF32x4SRelaxed
+            | Operator::I32x4TruncSatF32x4URelaxed
+            | Operator::I32x4TruncSatF64x2SZeroRelaxed
+            | Operator::I32x4TruncSatF64x2UZeroRelaxed => {
+                self.check_relaxed_simd_enabled()?;
+                self.pop_operand(Some(Type::V128))?;
+                self.push_operand(Type::V128)?;
+            }
             Operator::V128Bitselect => {
                 self.check_simd_enabled()?;
                 self.pop_operand(Some(Type::V128))?;
@@ -1683,6 +1741,16 @@ impl OperatorValidator {
                 self.pop_operand(Some(Type::V128))?;
                 self.push_operand(Type::V128)?;
             }
+            Operator::I8x16LaneSelect
+            | Operator::I16x8LaneSelect
+            | Operator::I32x4LaneSelect
+            | Operator::I64x2LaneSelect => {
+                self.check_relaxed_simd_enabled()?;
+                self.pop_operand(Some(Type::V128))?;
+                self.pop_operand(Some(Type::V128))?;
+                self.pop_operand(Some(Type::V128))?;
+                self.push_operand(Type::V128)?;
+            }
             Operator::V128AnyTrue
             | Operator::I8x16AllTrue
             | Operator::I8x16Bitmask
@@ -1719,6 +1787,12 @@ impl OperatorValidator {
                 self.pop_operand(Some(Type::V128))?;
                 self.push_operand(Type::V128)?;
             }
+            Operator::I8x16SwizzleRelaxed => {
+                self.check_relaxed_simd_enabled()?;
+                self.pop_operand(Some(Type::V128))?;
+                self.pop_operand(Some(Type::V128))?;
+                self.push_operand(Type::V128)?;
+            }
             Operator::I8x16Shuffle { ref lanes } => {
                 self.check_simd_enabled()?;
                 self.pop_operand(Some(Type::V128))?;
@@ -1978,9 +2052,9 @@ fn func_type_at<T: WasmModuleResources>(
 }
 
 fn tag_at<T: WasmModuleResources>(resources: &T, at: u32) -> OperatorValidatorResult<&T::FuncType> {
-    resources
-        .tag_at(at)
-        .ok_or_else(|| OperatorValidatorError::new("unknown tag: tag index out of bounds"))
+    resources.tag_at(at).ok_or_else(|| {
+        OperatorValidatorError::new(format!("unknown tag {}: tag index out of bounds", at))
+    })
 }
 
 enum Either<A, B> {
