@@ -362,6 +362,10 @@ impl V128 {
     pub fn bytes(&self) -> &[u8; 16] {
         &self.0
     }
+
+    pub fn i128(&self) -> i128 {
+        i128::from_le_bytes(self.0)
+    }
 }
 
 pub type SIMDLaneIndex = u8;
@@ -1249,4 +1253,21 @@ pub enum Operator<'a> {
     F64x2ConvertLowI32x4U,
     F32x4DemoteF64x2Zero,
     F64x2PromoteLowF32x4,
+    I8x16SwizzleRelaxed,
+    I32x4TruncSatF32x4SRelaxed,
+    I32x4TruncSatF32x4URelaxed,
+    I32x4TruncSatF64x2SZeroRelaxed,
+    I32x4TruncSatF64x2UZeroRelaxed,
+    F32x4FmaRelaxed,
+    F32x4FmsRelaxed,
+    F64x2FmaRelaxed,
+    F64x2FmsRelaxed,
+    I8x16LaneSelect,
+    I16x8LaneSelect,
+    I32x4LaneSelect,
+    I64x2LaneSelect,
+    F32x4MinRelaxed,
+    F32x4MaxRelaxed,
+    F64x2MinRelaxed,
+    F64x2MaxRelaxed,
 }
